@@ -6,11 +6,12 @@
 #include <EXTERN.h>
 #include <perl.h>
 #include <Python.h>
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
-
+#include "pycompat.h"
 #include "thrd_ctx.h"
 #include "svrv_object.h"
 #include "perlmodule.h"
@@ -107,11 +108,7 @@ pysvrv_dealloc(PySVRV *self)
 #endif
 
     ENTER_PYTHON;
-#if PY_MAJOR_VERSION >= 1 && PY_MINOR_VERSION >= 6
     PyObject_DEL(self);
-#else
-    PyMem_DEL(self);
-#endif
     ASSERT_LOCK_PYTHON;
 }
 
@@ -120,7 +117,7 @@ static PyObject*
 pysvrv_has_key(PySVRV *self, PyObject *args)
 {
     char *key;
-    int keylen;
+    Py_ssize_t keylen;
     int exists;
     dCTXP;
 
@@ -135,10 +132,9 @@ pysvrv_has_key(PySVRV *self, PyObject *args)
     exists = hv_exists((HV*)SvRV(self->rv), key, keylen);
 
     ENTER_PYTHON;
-    return PyInt_FromLong(exists);
+    return PyBool_FromLong(exists);
 }
 
-
 static PyObject*
 do_hash_kv(HV* hv, bool do_keys, bool do_values)
 {
@@ -172,7 +168,7 @@ do_hash_kv(HV* hv, bool do_keys, bool do_values)
 	    I32 klen;
 	    char *kstr = hv_iterkey(entry, &klen);
 	    ENTER_PYTHON;
-	    k = PyString_FromStringAndSize(kstr, klen);
+	    k = PyUnicode_FromStringAndSize(kstr, klen);
 	    if (k == NULL)
 		goto FAIL;
 	    ENTER_PERL;
@@ -219,6 +215,51 @@ FAIL:
     return NULL;
 }
 
+
+static int
+pysvrv_contains(PySVRV *self, PyObject *value)
+{
+    char *key;
+    Py_ssize_t keylen;
+    bool exists;
+    PyObject *found;
+    SV** svp;
+    dCTXP;
+
+    if (!PyUnicode_Check(value)) {
+	PyErr_SetString(PyExc_TypeError, "perl hash key must be string");
+	return -1;
+    }
+
+    ASSERT_LOCK_PYTHON;
+    CHECK_OWNED_PY;
+    key = PyUnicode_AsUTF8AndSize(value, &keylen);
+
+    ENTER_PERL;
+    SET_CUR_PERL;
+    assert(SvTYPE(SvRV(self->rv)) == SVt_PVHV);
+    exists = hv_exists((HV*)SvRV(self->rv), key, keylen);
+
+    ENTER_PYTHON;
+#if PY_MAJOR_VERSION < 3
+    if (svp) {
+	PERL_LOCK;
+	found = sv2pyo(*svp);
+	PERL_UNLOCK;
+    } else {
+	if (PyErr_Occurred()) {
+	    Py_DECREF(key);
+	    return -1;
+	}
+	return 0;
+    }
+
+    return PyObject_RichCompareBool(value, found, Py_EQ);
+#else
+    return exists;
+#endif
+}
+
 static PyObject*
 pysvrv_keys(PySVRV *self, PyObject *args)
 {
@@ -227,7 +268,7 @@ pysvrv_keys(PySVRV *self, PyObject *args)
     CHECK_OWNED_PY;
 
     ASSERT_LOCK_PYTHON;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
     assert(SvTYPE(SvRV(self->rv)) == SVt_PVHV);
     return do_hash_kv((HV*)SvRV(self->rv), TRUE, FALSE);
@@ -241,7 +282,7 @@ pysvrv_values(PySVRV *self, PyObject *args)
     CHECK_OWNED_PY;
 
     ASSERT_LOCK_PYTHON;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
     assert(SvTYPE(SvRV(self->rv)) == SVt_PVHV);
     return do_hash_kv((HV*)SvRV(self->rv), FALSE, TRUE);
@@ -255,7 +296,7 @@ pysvrv_items(PySVRV *self, PyObject *args)
     SET_CUR_PERL;
 #endif
     ASSERT_LOCK_PYTHON;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
     assert(SvTYPE(SvRV(self->rv)) == SVt_PVHV);
     return do_hash_kv((HV*)SvRV(self->rv), TRUE, TRUE);
@@ -348,7 +389,7 @@ pysvrv_clear(PySVRV *self, PyObject *args)
 
     ASSERT_LOCK_PYTHON;
     CHECK_OWNED_PY;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
 
     ENTER_PERL;
@@ -375,7 +416,7 @@ pysvrv_copy(PySVRV *self, PyObject *args)
 
     ASSERT_LOCK_PYTHON;
     CHECK_OWNED_PY;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
 
     ENTER_PERL;
@@ -460,7 +501,7 @@ array_splice(AV* av, I32 offset, I32 len, I32 newlen)
 
     ASSERT_LOCK_PERL;
     SET_CUR_PERL;
-/* #define SPLICE_DEBUG  /* */
+/* #define SPLICE_DEBUG  */
     asize = av_len(av) + 1;
     if (offset < 0)
 	offset += asize;
@@ -492,7 +533,7 @@ array_splice(AV* av, I32 offset, I32 len, I32 newlen)
     
     diff = newlen - len;
     if (newlen && !AvREAL(av) && AvREIFY(av))
-	av_reify(av);
+	Perl_av_reify(aTHX_ av);
 
 #ifdef SPLICE_DEBUG
     printf("splice(offset=%d, len=%d, diff=%d, after=%d, fill=%d, max=%d, pre=%d)\n",
@@ -712,8 +753,8 @@ pysvrv_extend(PySVRV *self, PyObject *args)
 {
     PyObject *o;
     AV* av;
-    int n, i;
-    STRLEN size;
+    int n;
+    STRLEN size, i;
     dCTXP;
 
     ASSERT_LOCK_PYTHON;
@@ -875,7 +916,7 @@ array_index(AV* av, PyObject *v)
 	    PERL_LOCK;
 	    x = sv2pyo(*svp);
 	    PERL_UNLOCK;
-	    cmp = PyObject_Compare(x, v);
+	    cmp = PyObject_RichCompareBool(x, v, Py_EQ);
 	    Py_DECREF(x);
 	    if (cmp == 0) {
 		ENTER_PERL;
@@ -965,7 +1006,7 @@ pysvrv_index(PySVRV *self, PyObject *args)
     }
 
     ASSERT_LOCK_PYTHON;
-    return PyInt_FromLong((long)index);
+    return PyLong_FromLong((long)index);
 }
 
 static PyObject*
@@ -1000,7 +1041,7 @@ pysvrv_count(PySVRV *self, PyObject *args)
 	    PERL_LOCK;
 	    x = sv2pyo(*svp);
 	    PERL_UNLOCK;
-	    cmp = PyObject_Compare(x, v);
+	    cmp = PyObject_RichCompareBool(x, v, Py_EQ);
 	    Py_DECREF(x);
 	    if (cmp == 0)
 		count++;
@@ -1014,7 +1055,7 @@ pysvrv_count(PySVRV *self, PyObject *args)
 	    count++;
     }
     ENTER_PYTHON;
-    return PyInt_FromLong((long)count);
+    return PyLong_FromLong((long)count);
 }
 
 static PyObject*
@@ -1026,7 +1067,7 @@ pysvrv_reverse(PySVRV *self, PyObject *args)
 
     ASSERT_LOCK_PYTHON;
     CHECK_OWNED_PY;
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
 
     ENTER_PERL;
@@ -1090,7 +1131,7 @@ pysvrv_av_alloc(PySVRV *self, PyObject *args)
     SET_CUR_PERL;
     CHECK_OWNED_PY;
 
-    if (!PyArg_NoArgs(args))
+    if (!PyArg_Parse(args, ""))
 	return NULL;
 
     ENTER_PERL;
@@ -1106,24 +1147,36 @@ pysvrv_av_alloc(PySVRV *self, PyObject *args)
     if (t == NULL) 
 	return NULL;
 
-    PyTuple_SetItem(t, 0, PyInt_FromLong(left));
-    PyTuple_SetItem(t, 1, PyInt_FromLong(middle));
-    PyTuple_SetItem(t, 2, PyInt_FromLong(right));
+    PyTuple_SetItem(t, 0, PyLong_FromLong(left));
+    PyTuple_SetItem(t, 1, PyLong_FromLong(middle));
+    PyTuple_SetItem(t, 2, PyLong_FromLong(right));
     ASSERT_LOCK_PYTHON;
     return t;
 }
 
 
-static PyMethodDef mapp_methods[] = {
+static PyMethodDef pysvrv_methods[] = {
+    /* mapp_methods */
     {"has_key",	(PyCFunction)pysvrv_has_key, METH_VARARGS},
-    {"keys",	(PyCFunction)pysvrv_keys,    0},
-    {"items",	(PyCFunction)pysvrv_items,   0},
-    {"values",	(PyCFunction)pysvrv_values,  0},
+    {"keys",	(PyCFunction)pysvrv_keys,    METH_NOARGS},
+    {"items",	(PyCFunction)pysvrv_items,   METH_NOARGS},
+    {"values",	(PyCFunction)pysvrv_values,  METH_NOARGS},
     {"update",	(PyCFunction)pysvrv_update,  METH_VARARGS},
-    {"clear",	(PyCFunction)pysvrv_clear,   0},
-    {"copy",	(PyCFunction)pysvrv_copy,    0},
+    {"clear",	(PyCFunction)pysvrv_clear,   METH_NOARGS},
+    {"copy",	(PyCFunction)pysvrv_copy,    METH_NOARGS},
     {"get",     (PyCFunction)pysvrv_get,     METH_VARARGS},
-  {NULL, NULL} /* sentinel */
+    /* list_methods */
+    {"append",	(PyCFunction)pysvrv_append,  METH_VARARGS},
+    {"insert",	(PyCFunction)pysvrv_insert,  METH_VARARGS},
+    {"extend",  (PyCFunction)pysvrv_extend,  METH_VARARGS},
+    {"pop",	(PyCFunction)pysvrv_pop,     METH_VARARGS},
+    {"remove",	(PyCFunction)pysvrv_remove,  METH_VARARGS},
+    {"index",	(PyCFunction)pysvrv_index,   METH_VARARGS},
+    {"count",	(PyCFunction)pysvrv_count,   METH_VARARGS},
+    {"reverse",	(PyCFunction)pysvrv_reverse, METH_NOARGS},
+    {"sort",	(PyCFunction)pysvrv_sort,    METH_VARARGS},
+    {"av_alloc",(PyCFunction)pysvrv_av_alloc,METH_NOARGS},
+    {NULL, NULL} /* sentinel */
 };
 
 static PyMethodDef list_methods[] = {
@@ -1134,34 +1187,40 @@ static PyMethodDef list_methods[] = {
     {"remove",	(PyCFunction)pysvrv_remove,  METH_VARARGS},
     {"index",	(PyCFunction)pysvrv_index,   METH_VARARGS},
     {"count",	(PyCFunction)pysvrv_count,   METH_VARARGS},
-    {"reverse",	(PyCFunction)pysvrv_reverse, 0},
+    {"reverse",	(PyCFunction)pysvrv_reverse, METH_NOARGS},
     {"sort",	(PyCFunction)pysvrv_sort,    METH_VARARGS},
-    {"av_alloc",(PyCFunction)pysvrv_av_alloc,0},
-  {NULL, NULL} /* sentinel */
+    {"av_alloc",(PyCFunction)pysvrv_av_alloc,METH_NOARGS},
+    {NULL, NULL} /* sentinel */
 };
 
 
 static PyObject*
-pysvrv_getattr(PySVRV *self, char *name)
+pysvrv_getattro(PySVRV *self, PyObject *nameobj)
 {
     PyObject *val;
     dCTXP;
+    char *name = "";
 
     ASSERT_LOCK_PYTHON;
     CHECK_OWNED_PY;
     SET_CUR_PERL;
 
+    if (PyUnicode_Check(nameobj))
+	name = PyUnicode_AsUTF8(nameobj);
+
     if (strcmp(name, "__wantarray__") == 0) {
 	if (self->gimme == G_VOID)
 	    val = Py_BuildValue(""); /* None */
 	else 
-	    val = PyInt_FromLong((long)(self->gimme == G_ARRAY));
+	    val = PyLong_FromLong((long)(self->gimme == G_ARRAY));
+	goto success;
     }
     else if (strcmp(name, "__methodname__") == 0) {
 	if (self->methodname)
-	    val = PyString_FromString(self->methodname);
+	    val = PyUnicode_FromString(self->methodname);
 	else
 	    val = Py_BuildValue(""); /* None */
+	goto success;
     }
     else if (strcmp(name, "__class__") == 0) {
 	SV *sv;
@@ -1170,19 +1229,21 @@ pysvrv_getattr(PySVRV *self, char *name)
 	if (SvOBJECT(sv)) {
 	    char *klass = HvNAME(SvSTASH(sv));
 	    ENTER_PYTHON;
-	    val = PyString_FromString(klass);
+	    val = PyUnicode_FromString(klass);
 	}
 	else {
 	    ENTER_PYTHON;
 	    val = Py_BuildValue("");
 	}
+	goto success;
     }
     else if (strcmp(name, "__type__") == 0) {
-	char *tmp;
+	const char *tmp;
 	ENTER_PERL;
 	tmp = sv_reftype(SvRV(self->rv), 0);
 	ENTER_PYTHON;
-	val = PyString_FromString(tmp);
+	val = PyUnicode_FromString(tmp);
+	goto success;
     }
     else if (strcmp(name, "__value__") == 0) {
 	SV *sv = SvRV(self->rv);
@@ -1191,20 +1252,22 @@ pysvrv_getattr(PySVRV *self, char *name)
 	case SVt_PVHV:
 	case SVt_PVCV:
 	    PyErr_SetString(PyExc_AttributeError, name);
-	    val = NULL;
+	    goto error;
 	    break;
 	default:
 	    PERL_LOCK;
 	    val = sv2pyo(sv); 
 	    PERL_UNLOCK;
 	}
+	goto success;
     }
     else if (strcmp(name, "__readonly__") == 0) {
-	val = PyInt_FromLong(SvREADONLY(SvRV(self->rv)) != 0);
+	val = PyLong_FromLong(SvREADONLY(SvRV(self->rv)) != 0);
+	goto success;
     }
     else if (self->methodname) {
 	PyErr_SetString(PyExc_AttributeError, name);
-	val = NULL;
+	goto error;
     }
     else if (SvOBJECT(SvRV(self->rv))) {
 	PySVRV *method_obj;
@@ -1225,32 +1288,44 @@ pysvrv_getattr(PySVRV *self, char *name)
 	}
 	PERL_UNLOCK;
 	val = (PyObject *)method_obj;
+	goto success;
     }
-    else if (SvTYPE(SvRV(self->rv)) == SVt_PVAV) {
-	val = Py_FindMethod(list_methods, (PyObject *)self, name);
+    else if (SvTYPE(SvRV(self->rv)) == SVt_PVAV || SvTYPE(SvRV(self->rv)) == SVt_PVHV) {
+	val = PyObject_GenericGetAttr((PyObject *)self, nameobj);
+	goto success;
     }
-    else if (SvTYPE(SvRV(self->rv)) == SVt_PVHV) {
-	val = Py_FindMethod(mapp_methods, (PyObject *)self, name);
+    else if (!(val = PyObject_GenericGetAttr((PyObject *)self, nameobj))) {
+	if (!PyErr_ExceptionMatches(PyExc_AttributeError))
+	    goto error;
+	else
+	    goto success;
     }
     else {
 	PyErr_SetString(PyExc_AttributeError, name);
-	val = NULL;
+	goto error;
     }
 
+error:
+    val = NULL;
+success:
     ASSERT_LOCK_PYTHON;
     return val;
 }
 
 static int
-pysvrv_setattr(PySVRV *self, char *name, PyObject *val)
+pysvrv_setattro(PySVRV *self, PyObject *nameobj, PyObject *val)
 {
     int status;
     dCTXP;
+    char *name = "";
 
     ASSERT_LOCK_PYTHON;
     CHECK_OWNED_INT;
     SET_CUR_PERL;
 
+    if (PyUnicode_Check(nameobj))
+	name = PyUnicode_AsUTF8(nameobj);
+
     if (strcmp(name, "__wantarray__") == 0) {
 	if (val == Py_None)
 	    self->gimme = G_VOID;
@@ -1259,12 +1334,12 @@ pysvrv_setattr(PySVRV *self, char *name, PyObject *val)
 	status = 0;
     }
     else if (strcmp(name, "__methodname__") == 0) {
-	if (PyString_Check(val)) {
+	if (PyUnicode_Check(val)) {
 	    PERL_LOCK;
 	    Safefree(self->methodname);
-	    New(998, self->methodname, PyString_GET_SIZE(val)+1, char);
-	    Copy(PyString_AS_STRING(val), self->methodname,
-		 PyString_GET_SIZE(val)+1, char);
+	    New(998, self->methodname, PyUnicode_GetLength(val)+1, char);
+	    Copy(PyUnicode_AsUTF8(val), self->methodname,
+		 PyUnicode_GetLength(val)+1, char);
 	    PERL_UNLOCK;
 	    status = 0;
 	}
@@ -1274,8 +1349,8 @@ pysvrv_setattr(PySVRV *self, char *name, PyObject *val)
 	}
     }
     else if (strcmp(name, "__class__") == 0) {
-	if (PyString_Check(val)) {
-	    char *klass = PyString_AsString(val);
+	if (PyUnicode_Check(val)) {
+	    char *klass = PyUnicode_AsUTF8(val);
 	    ENTER_PERL;
 	    sv_bless(self->rv, gv_stashpv(klass, 1));
 	    ENTER_PYTHON;
@@ -1380,7 +1455,7 @@ pysvrv_repr(PySVRV *self)
     sv_catpvn(tmp_sv, ">", 1);
     ENTER_PYTHON;
 
-    o = PyString_FromStringAndSize(SvPVX(tmp_sv), SvCUR(tmp_sv));
+    o = PyUnicode_FromStringAndSize(SvPVX(tmp_sv), SvCUR(tmp_sv));
     SvREFCNT_dec(tmp_sv);
 
     ASSERT_LOCK_PYTHON;
@@ -1412,11 +1487,11 @@ type_error(char *msg, SV* sv)
 }
 
 
-static int
+static Py_ssize_t
 pysvrv_length(PySVRV *self)
 {
     SV* sv;
-    int len;
+    Py_ssize_t len;
     dCTX;
 
     ASSERT_LOCK_PYTHON;
@@ -1443,7 +1518,7 @@ pysvrv_length(PySVRV *self)
 }
 
 static int
-pysvrv_nonzero(PySVRV *self)
+pysvrv_bool(PySVRV *self)
 {
     SV* sv;
     int v;
@@ -1471,7 +1546,7 @@ pysvrv_nonzero(PySVRV *self)
 
 
 static PyObject *
-pysvrv_item(PySVRV *self, int index)
+pysvrv_item(PySVRV *self, Py_ssize_t index)
 {
     SV* sv;
     PyObject *item;
@@ -1509,8 +1584,8 @@ pysvrv_subscript(PySVRV *self, PyObject *key)
     sv = SvRV(self->rv);
     if (SvTYPE(sv) == SVt_PVAV) {
 	I32 index;
-	if (PyInt_Check(key))
-	    index = PyInt_AsLong(key);
+	if (PyLong_Check(key))
+	    index = PyLong_AsLong(key);
 	else if (PyLong_Check(key)) {
 	    index = PyLong_AsLong(key);
 	    if (index == -1 && PyErr_Occurred())
@@ -1525,10 +1600,10 @@ pysvrv_subscript(PySVRV *self, PyObject *key)
     }
     else if (SvTYPE(sv) == SVt_PVHV) {
 	HV* hv = (HV*)sv;
-	if (PyString_Check(key)) {
+	if (PyUnicode_Check(key)) {
 	    SV** svp;
 	    ENTER_PERL;
-	    svp = hv_fetch(hv, PyString_AsString(key), PyString_Size(key), 0);
+	    svp = hv_fetch(hv, PyUnicode_AsUTF8(key), PyUnicode_GetLength(key), 0);
 	    if (svp) {
 		SvGETMAGIC(*svp);
 		PYTHON_LOCK;
@@ -1576,8 +1651,8 @@ pysvrv_ass_sub(PySVRV *self, PyObject *key, PyObject *val)
 	SV* val_sv;
 	SV** svp;
 
-	if (PyInt_Check(key))
-	    index = PyInt_AsLong(key);
+	if (PyLong_Check(key))
+	    index = PyLong_AsLong(key);
 	else if (PyLong_Check(key)) {
 	    index = PyLong_AsLong(key);
 	    if (index == -1 && PyErr_Occurred())
@@ -1621,9 +1696,9 @@ pysvrv_ass_sub(PySVRV *self, PyObject *key, PyObject *val)
     }
     else if (SvTYPE(sv) == SVt_PVHV) {
 	HV* hv = (HV*)sv;
-	if (PyString_Check(key)) {
-	    char *key_str = PyString_AsString(key);
-	    int   key_len = PyString_Size(key);
+	if (PyUnicode_Check(key)) {
+	    char *key_str = PyUnicode_AsUTF8(key);
+	    int   key_len = PyUnicode_GetLength(key);
 	    if (val) {
 		SV* val_sv;
 		SV** svp;
@@ -1766,7 +1841,7 @@ DONE:
 
 
 static PyObject *
-pysvrv_repeat(PySVRV *self, int n)
+pysvrv_repeat(PySVRV *self, Py_ssize_t n)
 {
     SV* sv;
     PyObject *pyo_res;
@@ -1833,7 +1908,7 @@ pysvrv_repeat(PySVRV *self, int n)
 
 
 static PyObject *
-pysvrv_slice(PySVRV *self, int ilow, int ihigh)
+pysvrv_slice(PySVRV *self, Py_ssize_t ilow, Py_ssize_t ihigh)
 {
     SV* sv;
     PyObject *pyo_res;
@@ -1899,7 +1974,7 @@ pysvrv_slice(PySVRV *self, int ilow, int ihigh)
 }
 
 static int
-pysvrv_ass_slice(PySVRV *self, int ilow, int ihigh, PyObject *v)
+pysvrv_ass_slice(PySVRV *self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject *v)
 {
     SV* sv;
     int status;  /* return value */
@@ -1978,14 +2053,16 @@ static PyNumberMethods pysvrv_as_number = {
 	0,	/*nb_add*/
 	0,	/*nb_subtract*/
 	0,	/*nb_multiply*/
-	0,	/*nb_divide*/
+#if PY_MAJOR_VERSION < 3
+	0, 	/*nb_divide*/
+#endif
 	0,	/*nb_remainder*/
 	0,	/*nb_divmod*/
 	0,	/*nb_power*/
 	0,	/*nb_negative*/
 	0,	/*nb_positive*/
 	0,	/*nb_absolute*/
-	(inquiry)pysvrv_nonzero,	/*nb_nonzero*/
+	(inquiry)pysvrv_bool,	/*nb_bool*/
 	0,	/*nb_invert*/
 	0,	/*nb_lshift*/
 	0,	/*nb_rshift*/
@@ -2001,20 +2078,26 @@ static PyNumberMethods pysvrv_as_number = {
 };
 
 static PyMappingMethods pysvrv_as_mapping = {
-    (inquiry)pysvrv_length, /* mp_length */
+    (lenfunc)pysvrv_length, /* mp_length */
     (binaryfunc)pysvrv_subscript, /* mp_subscript */
     (objobjargproc)pysvrv_ass_sub, /* mp_ass_subscript */
 };
 
 static PySequenceMethods pysvrv_as_sequence = {
-    (inquiry)pysvrv_length, /*sq_length*/
+#if PY_MAJOR_VERSION >= 3
+    .sq_length = (lenfunc)pysvrv_length,
+    .sq_concat = (binaryfunc)pysvrv_concat,
+    .sq_repeat = (ssizeargfunc)pysvrv_repeat,
+    .sq_item = (ssizeargfunc)pysvrv_item,
+    .sq_contains = (objobjproc)pysvrv_contains,
+#else
+    (lenfunc)pysvrv_length, /*sq_length*/
     (binaryfunc)pysvrv_concat, /*sq_concat*/
-    (intargfunc)pysvrv_repeat, /*sq_repeat*/
-    (intargfunc)pysvrv_item, /*sq_item*/
-    (intintargfunc)pysvrv_slice, /*sq_slice*/
+    (ssizeargfunc)pysvrv_repeat, /*sq_repeat*/
+    (ssizeargfunc)pysvrv_item, /*sq_item*/
+    (ssizessizeargfunc)pysvrv_slice, /*sq_slice*/
     0, /*sq_ass_item*/
-    (intintobjargproc)pysvrv_ass_slice, /*sq_ass_slice*/
-#if PY_MAJOR_VERSION >= 1 && PY_MINOR_VERSION >= 6
+    (ssizessizeobjargproc)pysvrv_ass_slice, /*sq_ass_slice*/
     0, /*sq_contains*/
 #endif
 };
@@ -2022,22 +2105,18 @@ static PySequenceMethods pysvrv_as_sequence = {
 
 //XXX must compile as a C++ file on Windows
 PyTypeObject SVRVtype = {
-    PyObject_HEAD_INIT(&PyType_Type)
-    0,			         /* Number of items for varobject */
-    "perl ref",		         /* Name of this type */
-    sizeof(PyTypeObject),	 /* Basic object size */
-    0,			         /* Item size for varobject */
-    (destructor)pysvrv_dealloc,  /*tp_dealloc*/
-    0,                           /*tp_print*/
-    (getattrfunc)pysvrv_getattr, /*tp_getattr*/
-    (setattrfunc)pysvrv_setattr, /*tp_setattr*/
-    0,                           /*tp_compare*/
-    (reprfunc)pysvrv_repr,       /*tp_repr*/
-    &pysvrv_as_number,	         /*tp_as_number*/
-    &pysvrv_as_sequence,         /*tp_as_sequence*/
-    &pysvrv_as_mapping,	         /*tp_as_mapping*/
-    0,                           /*tp_hash*/
-    (ternaryfunc)pysvrv_call,    /*tp_call*/
+    PyVarObject_HEAD_INIT(&PyType_Type, 0)
+    .tp_name = "perl ref",
+    .tp_basicsize = sizeof(PyTypeObject),
+    .tp_dealloc = (destructor)pysvrv_dealloc,
+    .tp_repr = (reprfunc)pysvrv_repr,
+    .tp_as_number = &pysvrv_as_number,
+    .tp_as_sequence = &pysvrv_as_sequence,
+    .tp_as_mapping = &pysvrv_as_mapping,
+    .tp_call = (ternaryfunc)pysvrv_call,
+    .tp_getattro = (getattrofunc)pysvrv_getattro,
+    .tp_setattro = (setattrofunc)pysvrv_setattro,
+    .tp_methods = pysvrv_methods,
 };
 
 #ifdef __cplusplus
